/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#include "DHT22.h"


/*******************************************************************************
* Function Name: Fraction_Convert
****************************************************************************//**
*
* Converts a 8 bit binary number into a fractional decimal value
*
* \param num
* 8 bit binary value
*
* \return
* Fractional decimal value of the binary number
*
*******************************************************************************/
float Fraction_Convert(uint8_t num)
{
    float fraction = 0;
    int unit = 0;
    for( int i = 0; i<8; i++)
    {
        unit = num & 1;
        num = num>>1;
        fraction = fraction + unit * pow(2, -(1+i));
    }
    return fraction;
}

/* Function to read high and low pulse values */
uint32_t expectPulse(uint32_t pinState){
    
    uint32_t count = 0;
    
    if(pinState == 0){
        
        while(DATA_Read() == 0UL){
            count++;
            
            if(count > MIN_INTERVAL){
                count = TIMEOUT;
                break;
            }
        }
    }else{
        
        while(DATA_Read() == 1UL){
            count++;
            if(count > MIN_INTERVAL){
                count = TIMEOUT;
                break;
            }
        }
    }
    
    return count;
}

/* Function to set read sensor values */
 CY_DHT22_RSLT_T DHT_Read(uint8* humidity, uint8* temperature){
    
    int highCycle = 0;
    int lowCycle = 0;
    uint32_t cycles[80];
    int timeout = 0;
    
    for(int i = 0; i < 5; i++){
        data[i] = 0;
    }
    
    // Now start reading the data line to get the value from the DHT sensor.
    
    // Turn off interrupts temporarily because the next sections
    // are timing critical and we don't want any interruptions.
    //uint8_t state = CyEnterCriticalSection();
    
    DATA_SetDriveMode(DATA_DM_STRONG);
    DATA_Write(0);
    
    DATA_Write(LOW);
    CyDelay(RETENTION_TIME);
    
    DATA_Write(HIGH);
    
    DATA_SetDriveMode(DATA_DM_RES_UP);
    
    while(DATA_Read() == 1UL){
        timeout++;
            if(timeout > MIN_INTERVAL){
                timeout = TIMEOUT;
                break;
            }
    };
    
    // First expect a low signal for ~80 microseconds followed by a high signal
    // for ~80 microseconds again.
    while(DATA_Read() == 0UL){
        timeout++;
            if(timeout > MIN_INTERVAL*2){
                timeout = TIMEOUT;
                break;
        }
    };
    
    while(DATA_Read() == 1UL){
        timeout++;
            if(timeout > MIN_INTERVAL*3){
                timeout = TIMEOUT;
                break;
        }
    };
    
    // Error handling when sensor doesn't respond
    if(timeout == TIMEOUT){
        return false;
    }
    
    // Now read the 40 bits sent by the sensor.  Each bit is sent as a 50
    // microsecond low pulse followed by a variable length high pulse.  If the
    // high pulse is ~28 microseconds then it's a 0 and if it's ~70 microseconds
    // then it's a 1.  We measure the cycle count of the initial 50us low pulse
    // and use that to compare to the cycle count of the high pulse to determine
    // if the bit is a 0 (high state cycle count < low state cycle count), or a
    // 1 (high state cycle count > low state cycle count). Note that for speed
    // all the pulses are read into a array and then examined in a later step.
    for(int j = 0; j < 80; j+=2){
        cycles[j] = expectPulse(0UL);
        cycles[j+1] = expectPulse(1UL);
    }
    
    //CyExitCriticalSection(state);
    
    // Timing critical code is now complete.
    
    // Inspect pulses and determine which ones are 0 (high state cycle count < low
    // state cycle count), or 1 (high state cycle count > low state cycle count).
    for(int j = 0; j < 40; j++){
        
        highCycle = cycles[2*j+1];
        lowCycle = cycles[2*j];
        
        // Error handling when sensor doesn't respond
        if((lowCycle == TIMEOUT) || (highCycle == TIMEOUT)){
            return false;
        }
        
        data[j/8] <<= 1;
        
        // Now compare the low and high cycle times to see if the bit is a 0 or 1.
        if(highCycle > lowCycle){
            data[j/8] |=  1;
        }
    }
    
    //Revert line to default state
    DATA_SetDriveMode(DATA_DM_STRONG);
    DATA_Write(HIGH);
    
    /* Reuse of temp variable used to verify checksum */
    /* Checksum is calculated by adding all 4 bytes */
    uint8 checksum = (data[0] + data[1] + data[2] + data[3]) & 0xFF;
    
    if((checksum == data[4]) && (data[4] != 0))
    {
        /* Pass the temperature and humidity data only when checksum is matched and it is not equal to 0 */
        *humidity = (int)data[0]; //+ Fraction_Convert(data[1]);
        *temperature = (int)data[2]; //+ Fraction_Convert(data[3]);
        
        return true;
    }
    else 
        return false; /* Incorrect value */
    
}

/* [] END OF FILE */
